using System;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace RSS {

	public class Channel {
		public string Title;
		public string Link;
		public string Copyright;
		public string Description;
	}

	public class Item {
		public string Title;
		public string Link;
		public string Description;
		public string ContentEncoded;
		public DateTime Date;
	}

	public class Parser : YASAX.Parser {

		enum Tag {
			Unknown,
			rdf,
			rdf_channel,
			rdf_channel_title,
			rdf_channel_link,
			rdf_channel_copyright,
			rdf_channel_description,
			rdf_item,
			rdf_item_link,
			rdf_item_title,
			rdf_item_contentencoded,
			rdf_item_description,
			rdf_item_date,
			rss,
			rss_channel,
			rss_channel_title,
			rss_channel_link,
			rss_channel_description,
			rss_channel_copyright,
			rss_channel_item,
			rss_channel_item_link,
			rss_channel_item_title,
			rss_channel_item_description,
			rss_channel_item_content,
			rss_channel_item_pubDate,
		}

		List<Tag> _tag_stack;
		List<Item> _items;
		Item _temp_item;
		Channel _temp_channel;
		StringBuilder _text;

		protected override void StartElement(string name, ListDictionary attr)
		{
			Tag tag = Tag.Unknown;
			if (_tag_stack.Count == 0) {
				switch (name) {
				case "rdf:RDF":
					tag = Tag.rdf;
					break;
				case "rss":
					tag = Tag.rss;
					break;
				}
			} else {
				Tag lasttag = _tag_stack[_tag_stack.Count - 1];
				if (lasttag == Tag.rdf) {
					switch (name) {
					case "channel":
						tag = Tag.rdf_channel;
						break;
					case "item":
						_temp_item = new Item();
						tag = Tag.rdf_item;
						break;
					}
				} else if (lasttag == Tag.rss) {
					switch (name) {
					case "channel":
						_temp_channel = new Channel();
						tag = Tag.rss_channel;
						break;
					}
				} else if (lasttag == Tag.rss_channel) {
					switch (name) {
					case "title":
						_text = new StringBuilder();
						tag = Tag.rss_channel_title;
						break;
					case "link":
						_text = new StringBuilder();
						tag = Tag.rss_channel_link;
						break;
					case "copyright":
						_text = new StringBuilder();
						tag = Tag.rss_channel_copyright;
						break;
					case "description":
						_text = new StringBuilder();
						tag = Tag.rss_channel_description;
						break;
					case "item":
						_temp_item = new Item();
						tag = Tag.rss_channel_item;
						break;
					}
				} else if (lasttag == Tag.rdf_channel) {
					switch (name) {
					case "title":
						_text = new StringBuilder();
						tag = Tag.rdf_channel_title;
						break;
					case "link":
						_text = new StringBuilder();
						tag = Tag.rdf_channel_link;
						break;
					case "copyright":
						_text = new StringBuilder();
						tag = Tag.rdf_channel_copyright;
						break;
					case "description":
						_text = new StringBuilder();
						tag = Tag.rdf_channel_description;
						break;
					}
				} else if (lasttag == Tag.rdf_item) {
					switch (name) {
					case "title":
						_text = new StringBuilder();
						tag = Tag.rdf_item_title;
						break;
					case "link":
						_text = new StringBuilder();
						tag = Tag.rdf_item_link;
						break;
					case "description":
						_text = new StringBuilder();
						tag = Tag.rdf_item_description;
						break;
					case "content:encoded":
						_text = new StringBuilder();
						tag = Tag.rdf_item_contentencoded;
						break;
					case "dc:date":
						_text = new StringBuilder();
						tag = Tag.rdf_item_date;
						break;
					}
				} else if (lasttag == Tag.rss_channel_item) {
					switch (name) {
					case "title":
						_text = new StringBuilder();
						tag = Tag.rss_channel_item_title;
						break;
					case "link":
						_text = new StringBuilder();
						tag = Tag.rss_channel_item_link;
						break;
					case "description":
						_text = new StringBuilder();
						tag = Tag.rss_channel_item_description;
						break;
					case "pubDate":
						_text = new StringBuilder();
						tag = Tag.rss_channel_item_pubDate;
						break;
					}
				}
			}
			_tag_stack.Add(tag);
		}

		protected override void EndElement(string name)
		{
			int n = _tag_stack.Count;
			if (n > 0) {
				Tag tag = _tag_stack[n - 1];
				_tag_stack.RemoveAt(n - 1);
				switch (tag) {
				case Tag.rss_channel_title:
				case Tag.rdf_channel_title:
					_temp_channel.Title = _text.ToString().Trim();
					break;
				case Tag.rss_channel_link:
				case Tag.rdf_channel_link:
					_temp_channel.Link = _text.ToString().Trim();
					break;
				case Tag.rss_channel_copyright:
				case Tag.rdf_channel_copyright:
					_temp_channel.Copyright = _text.ToString().Trim();
					break;
				case Tag.rss_channel_description:
				case Tag.rdf_channel_description:
					_temp_channel.Description = _text.ToString().Trim();
					break;
				case Tag.rdf_item_title:
				case Tag.rss_channel_item_title:
					_temp_item.Title = _text.ToString().Trim();
					_text = null;
					break;
				case Tag.rdf_item_link:
				case Tag.rss_channel_item_link:
					_temp_item.Link = _text.ToString().Trim();
					_text = null;
					break;
				case Tag.rdf_item_description:
				case Tag.rss_channel_item_description:
					_temp_item.Description = _text.ToString().Trim();
					_text = null;
					break;
				case Tag.rdf_item_contentencoded:
					_temp_item.ContentEncoded = _text.ToString().Trim();
					_text = null;
					break;
				case Tag.rdf_item_date:
				case Tag.rss_channel_item_pubDate:
					try {
						_temp_item.Date = DateTime.Parse(_text.ToString().Trim());
					} catch {
					}
					_text = null;
					break;
				case Tag.rdf_item:
				case Tag.rss_channel_item:
					_items.Add(_temp_item);
					break;
				}
			}
		}

		protected override void Characters(string text)
		{
			if (_text != null) {
				_text.Append(text);
			}
		}

		public string Message;

		public bool Parse(string path, out RSS.Channel channel, out List<RSS.Item> items)
		{
			channel = null;
			items = null;
			try {
				Message = null;
				_tag_stack = new List<Tag>();
				_temp_channel = new Channel();
				_items = new List<Item>();
				System.Net.WebClient client = new System.Net.WebClient();
				using (Stream s = client.OpenRead(path)) {
					//FileStream s = File.Open(path, FileMode.Open);
					Parse(s);
					s.Close();
				}
				channel = _temp_channel;
				items = _items;
				return true;
			} catch (YASAX.MyException e) {
				Message = e.Message;
				return false;
			} catch (Exception e) {
				Message = e.Message;
				return false;
			} finally {
				_temp_channel = null;
				_items = null;
				_text = null;
				_temp_item = null;
				_tag_stack = null;
			}
		}
	}
}
